-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "MG_LineWidthMagnet"

-- **************************************************
-- General information about this script
-- **************************************************

MG_LineWidthMagnet = {}

MG_LineWidthMagnet.BASE_STR = 2145

function MG_LineWidthMagnet:Name()
	return "Line Width Magnet"
end

function MG_LineWidthMagnet:Version()
	return "6.0"
end

function MG_LineWidthMagnet:Description()
	return "Change line width in circle. Up/Down with falloff, L/R constant. Shift L/R changes diameter"
end

function MG_LineWidthMagnet:Creator()
	return "Lost Marble & Ramn Lpez & Mike Green amendments"
end

function MG_LineWidthMagnet:UILabel()
	return(MOHO.Localize(self.BASE_STR + 1, "Magneto Linea"))
end

---[[
function MG_LineWidthMagnet:LoadPrefs(prefs)
	self.magnetRadius = prefs:GetFloat("MG_LineWidthMagnet.magnetRadius", 0.5)
	self.closest = prefs:GetBool("MG_LineWidthMagnet.closest", true)
end--]]

---[[
function MG_LineWidthMagnet:SavePrefs(prefs)
	prefs:SetFloat("MG_LineWidthMagnet.magnetRadius", self.magnetRadius)
	prefs:SetBool("MG_LineWidthMagnet.closest", self.closest)
end--]]

-- **************************************************
-- Recurring values
-- **************************************************

MG_LineWidthMagnet.newMethod = false
MG_LineWidthMagnet.magnetRadius = 0.5
MG_LineWidthMagnet.dragging = false
MG_LineWidthMagnet.dragVec = LM.Vector2:new_local()
MG_LineWidthMagnet.closest = true

MG_LineWidthMagnet.minVec = LM.Vector2:new_local()
MG_LineWidthMagnet.maxVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function MG_LineWidthMagnet:IsEnabled(moho)
	if (moho:CountPoints() > 0) then
		return true
	end
	return false
end

function MG_LineWidthMagnet:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.dragging = true
	self.dragVec:Set(mouseEvent.vec)

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.scaleList = {}

-- If Closest is on, select all points linked to the closest point to the cursor

 	if (self.closest) then
 		mesh:SelectNone()
 		mypointid = mesh:ClosestPoint(mouseEvent.vec)
        mypoint = mesh:Point(mypointid)
		mypoint.fSelected = true
        mesh:SelectConnected()
	else
		mesh:SelectAll()
 	end
	self.selList = MOHO.SelectedPointList(mesh)


	for i, pt in self.selList do
--		local pt = mesh:Point(i)
		local v = pt.fPos - mouseEvent.vec
		local magInfluence = v:Mag()
		
		if (pt.fWidth.value < 0) then -- Added by R.e.M, form here...
			pt.fWidth:SetValue(moho.frame, 0)
		end
		pt.fTempWidth = pt.fWidth.value -- ...to here.

		if (self.newMethod) then
			-- new method (more similar to how bones work)
			-- however, the "strength" is set to a constant 0.5 - the user would need to be able to change this
			magInfluence = magInfluence * magInfluence
			magInfluence = 0.5 / (1 + magInfluence * magInfluence * 100)
			if (magInfluence > 0.001 and ((self.closest and pt.fSelected))) then -- or not self.closest)) then
				pt.fSelected = true
				table.insert(self.scaleList, magInfluence)
			else
				pt.fSelected = false
			end
		else
			-- old method
			if (magInfluence <= self.magnetRadius) then
				-- mark the point as selected, and compute the magnet's influence over it
				pt.fSelected = true
				magInfluence = magInfluence / self.magnetRadius
				table.insert(self.scaleList, LM.Slerp(magInfluence, 1, 0))
			else
				pt.fSelected = false
			end
		end

	end

	mesh:PrepMovePoints()
	self.selList = MOHO.SelectedPointList(mesh)

	mouseEvent.view:DrawMe()
end

function MG_LineWidthMagnet:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.dragVec:Set(mouseEvent.vec)
	local offset = mouseEvent.vec - mouseEvent.startVec

	if(mouseEvent.shiftKey) then
		self.magnetRadius = LM.Clamp(self.magnetRadius + offset.x,0,100)
		mouseEvent.startVec:Set(mouseEvent.vec)
		mouseEvent.view:DrawMe()
		self:UpdateWidgets(moho)
		return;
	end

	local maxWidth = moho:PixelToDoc(128) --Added by R.e.M

	for i, pt in self.selList do
		local test = pt.fTempWidth + offset.x + offset.y * self.scaleList[i]
		pt.fWidth.value = LM.Clamp(test, 0, maxWidth)
		pt.fWidth:StoreValue()
	end

--[[First method, more similar to original Magnet tool:
	for i, pt in self.selList do
		pt.fWidth:SetValue(moho.frame, pt.fTempWidth + offset.x + offset.y * self.scaleList[i])
		--pt.fWidth.value = pt.fTempWidth + offset.x * self.scaleList[i]
	end--]]


	mouseEvent.view:DrawMe()
	self:UpdateWidgets(moho)
end

function MG_LineWidthMagnet:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.dragging = false

	self.selList = nil
	self.scaleList = nil
	moho.layer:UpdateCurFrame()
	moho:NewKeyframe(CHANNEL_WIDTH)
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

function MG_LineWidthMagnet:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

function MG_LineWidthMagnet:DrawMe(moho, view)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (not self.dragging) then
		return
	end

	local g = view:Graphics()
	local matrix = LM.Matrix:new_local()

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	g:Push()
	g:ApplyMatrix(matrix)

	g:SetColor(255, 0, 0, 128)
	g:SetSmoothing(true)
	g:FillCircle(self.dragVec, self.magnetRadius)
	g:SetSmoothing(false)

	g:Pop()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

MG_LineWidthMagnet.CHANGE = MOHO.MSG_BASE
MG_LineWidthMagnet.DUMMY = MOHO.MSG_BASE + 1
MG_LineWidthMagnet.RESET = MOHO.MSG_BASE + 2
MG_LineWidthMagnet.SELECTITEM = MOHO.MSG_BASE + 3
MG_LineWidthMagnet.CLOSEST = MOHO.MSG_BASE + 4

function MG_LineWidthMagnet:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu(MOHO.Localize(self.BASE_STR + 2, "Select Group"))

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize(self.BASE_STR + 3, "Magnet radius")))
	self.radius = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_UFLOAT)
	self.radius:SetWheelInc(0.1)
	layout:AddChild(self.radius)

	layout:AddChild(LM.GUI.Button(MOHO.Localize(MOHO.STR_RESET, "Reset"), self.RESET))
	self.closestCheck = LM.GUI.CheckBox("Use closest line", self.CLOSEST)
	layout:AddChild(self.closestCheck)
end

function MG_LineWidthMagnet:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.radius:SetValue(self.magnetRadius)
	self.closestCheck:SetValue(self.closest)
end

function MG_LineWidthMagnet:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.RESET) then
		self.magnetRadius = 0.5
	elseif (msg == self.CHANGE) then
		self.magnetRadius = self.radius:FloatValue()
		if (self.magnetRadius < 0.001) then
			self.magnetRadius = 0.001
		end
	elseif (msg == self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	elseif (msg >= self.CLOSEST) then
		self.closest = self.closestCheck:Value()
		moho:UpdateUI()
	end
end
